@@ -1,7 +1,7 @@
-#[cfg(not(feature = "async-mio"))]
+#[cfg(all(unix, not(feature = "async-mio")))]
 use ssip_client::{fifo, ClientName, ClientResult};
 
-#[cfg(not(feature = "async-mio"))]
+#[cfg(all(unix, not(feature = "async-mio")))]
 fn main() -> ClientResult<()> {
     let mut client = fifo::Builder::new().build()?;
     client
@@ -19,7 +19,12 @@ fn main() -> ClientResult<()> {
     Ok(())
 }
 
-#[cfg(feature = "async-mio")]
+#[cfg(all(unix, feature = "async-mio"))]
 fn main() {
     println!("see async_mio_loop for an example of asynchronous client.");
 }
+
+#[cfg(not(unix))]
+fn main() {
+    println!("example only available on unix.");
+}
