@@ -11,16 +11,17 @@ use std::io::{self, Read, Write};
 
 use crate::constants::*;
 use crate::protocol::{
-    flush_lines, parse_event_id, parse_single_value, parse_synthesis_voices, write_lines,
+    flush_lines, parse_event_id, parse_single_integer, parse_single_value, parse_typed_lines,
+    write_lines,
 };
 use crate::types::*;
 
 // Trick to have common implementation for std and mio streams..
 #[cfg(all(not(feature = "async-mio"), unix))]
-pub(crate) use std::os::unix::io::AsRawFd as Source;
+pub use std::os::unix::io::AsRawFd as Source;
 
 #[cfg(feature = "async-mio")]
-pub(crate) use mio::event::Source;
+pub use mio::event::Source;
 
 /// Convert boolean to ON or OFF
 fn on_off(value: bool) -> &'static str {
@@ -69,68 +70,84 @@ pub enum Request {
     GetPitch,
     SetVolume(ClientScope, i8),
     GetVolume,
-    SetPauseContext(ClientScope, u8),
-    SetHistory(ClientScope, bool),
+    SetPauseContext(ClientScope, u32),
     SetNotification(NotificationType, bool),
+    // Blocks
     Begin,
     End,
+    // History
+    SetHistory(ClientScope, bool),
+    HistoryGetClients,
+    HistoryGetClientId,
+    HistoryGetClientMsgs(ClientScope, u32, u32),
+    HistoryGetLastMsgId,
+    HistoryGetMsg(MessageId),
+    HistoryCursorGet,
+    HistoryCursorSet(ClientScope, HistoryPosition),
+    HistoryCursorMove(CursorDirection),
+    HistorySpeak(MessageId),
+    HistorySort(SortDirection, SortKey),
+    HistorySetShortMsgLength(u32),
+    HistorySetMsgTypeOrdering(Vec<Ordering>),
+    HistorySearch(ClientScope, String),
+    // Misc.
     Quit,
 }
 
 #[derive(Debug)]
 /// Response from SSIP server.
 pub enum Response {
-    LanguageSet,                         // 201
-    PrioritySet,                         // 202
-    RateSet,                             // 203
-    PitchSet,                            // 204
-    PunctuationSet,                      // 205
-    CapLetRecognSet,                     // 206
-    SpellingSet,                         // 207
-    ClientNameSet,                       // 208
-    VoiceSet,                            // 209
-    Stopped,                             // 210
-    Paused,                              // 211
-    Resumed,                             // 212
-    Canceled,                            // 213
-    TableSet,                            // 215
-    OutputModuleSet,                     // 216
-    PauseContextSet,                     // 217
-    VolumeSet,                           // 218
-    SsmlModeSet,                         // 219
-    NotificationSet,                     // 220
-    PitchRangeSet,                       // 263
-    DebugSet,                            // 262
-    HistoryCurSetFirst,                  // 220
-    HistoryCurSetLast,                   // 221
-    HistoryCurSetPos,                    // 222
-    HistoryCurMoveFor,                   // 223
-    HistoryCurMoveBack,                  // 224
-    MessageQueued,                       // 225,
-    SoundIconQueued,                     // 226
-    MessageCanceled,                     // 227
-    ReceivingData,                       // 230
-    Bye,                                 // 231
-    HistoryClientListSent(Vec<String>),  // 240
-    HistoryMsgsListSent(Vec<String>),    // 241
-    HistoryLastMsg(String),              // 242
-    HistoryCurPosRet(String),            // 243
-    TableListSent(Vec<String>),          // 244
-    HistoryClientIdSent(String),         // 245
-    MessageTextSent,                     // 246
-    HelpSent(Vec<String>),               // 248
-    VoicesListSent(Vec<SynthesisVoice>), // 249
-    OutputModulesListSent(Vec<String>),  // 250
-    Get(String),                         // 251
-    InsideBlock,                         // 260
-    OutsideBlock,                        // 261
-    NotImplemented,                      // 299
-    EventIndexMark(EventId, String),     // 700
-    EventBegin(EventId),                 // 701
-    EventEnd(EventId),                   // 702
-    EventCanceled(EventId),              // 703
-    EventPaused(EventId),                // 704
-    EventResumed(EventId),               // 705
+    LanguageSet,                                     // 201
+    PrioritySet,                                     // 202
+    RateSet,                                         // 203
+    PitchSet,                                        // 204
+    PunctuationSet,                                  // 205
+    CapLetRecognSet,                                 // 206
+    SpellingSet,                                     // 207
+    ClientNameSet,                                   // 208
+    VoiceSet,                                        // 209
+    Stopped,                                         // 210
+    Paused,                                          // 211
+    Resumed,                                         // 212
+    Canceled,                                        // 213
+    TableSet,                                        // 215
+    OutputModuleSet,                                 // 216
+    PauseContextSet,                                 // 217
+    VolumeSet,                                       // 218
+    SsmlModeSet,                                     // 219
+    NotificationSet,                                 // 220
+    PitchRangeSet,                                   // 263
+    DebugSet,                                        // 262
+    HistoryCurSetFirst,                              // 220
+    HistoryCurSetLast,                               // 221
+    HistoryCurSetPos,                                // 222
+    HistoryCurMoveFor,                               // 223
+    HistoryCurMoveBack,                              // 224
+    MessageQueued,                                   // 225,
+    SoundIconQueued,                                 // 226
+    MessageCanceled,                                 // 227
+    ReceivingData,                                   // 230
+    Bye,                                             // 231
+    HistoryClientListSent(Vec<HistoryClientStatus>), // 240
+    HistoryMsgsListSent(Vec<String>),                // 241
+    HistoryLastMsg(String),                          // 242
+    HistoryCurPosRet(String),                        // 243
+    TableListSent(Vec<String>),                      // 244
+    HistoryClientIdSent(ClientId),                   // 245
+    MessageTextSent,                                 // 246
+    HelpSent(Vec<String>),                           // 248
+    VoicesListSent(Vec<SynthesisVoice>),             // 249
+    OutputModulesListSent(Vec<String>),              // 250
+    Get(String),                                     // 251
+    InsideBlock,                                     // 260
+    OutsideBlock,                                    // 261
+    NotImplemented,                                  // 299
+    EventIndexMark(EventId, String),                 // 700
+    EventBegin(EventId),                             // 701
+    EventEnd(EventId),                               // 702
+    EventCanceled(EventId),                          // 703
+    EventPaused(EventId),                            // 704
+    EventResumed(EventId),                           // 705
 }
 
 macro_rules! send_one_line {
@@ -281,6 +298,45 @@ impl<S: Read + Write + Source> Client<S> {
             }
             Request::Begin => send_one_line!(self, "BLOCK BEGIN"),
             Request::End => send_one_line!(self, "BLOCK END"),
+            Request::HistoryGetClients => send_one_line!(self, "HISTORY GET CLIENT_LIST"),
+            Request::HistoryGetClientId => send_one_line!(self, "HISTORY GET CLIENT_ID"),
+            Request::HistoryGetClientMsgs(scope, start, number) => send_one_line!(
+                self,
+                "HISTORY GET CLIENT_MESSAGES {} {}_{}",
+                scope,
+                start,
+                number
+            ),
+            Request::HistoryGetLastMsgId => send_one_line!(self, "HISTORY GET LAST"),
+            Request::HistoryGetMsg(id) => send_one_line!(self, "HISTORY GET MESSAGE {}", id),
+            Request::HistoryCursorGet => send_one_line!(self, "HISTORY CURSOR GET"),
+            Request::HistoryCursorSet(scope, pos) => {
+                send_one_line!(self, "HISTORY CURSOR SET {} {}", scope, pos)
+            }
+            Request::HistoryCursorMove(direction) => {
+                send_one_line!(self, "HISTORY CURSOR {}", direction)
+            }
+            Request::HistorySpeak(id) => send_one_line!(self, "HISTORY SAY {}", id),
+            Request::HistorySort(direction, key) => {
+                send_one_line!(self, "HISTORY SORT {} {}", direction, key)
+            }
+            Request::HistorySetShortMsgLength(length) => {
+                send_one_line!(self, "HISTORY SET SHORT_MESSAGE_LENGTH {}", length)
+            }
+            Request::HistorySetMsgTypeOrdering(ordering) => {
+                send_one_line!(
+                    self,
+                    "HISTORY SET MESSAGE_TYPE_ORDERING \"{}\"",
+                    ordering
+                        .iter()
+                        .map(|x| x.to_string())
+                        .collect::<Vec<String>>()
+                        .join(" ")
+                )
+            }
+            Request::HistorySearch(scope, condition) => {
+                send_one_line!(self, "HISTORY SEARCH {} \"{}\"", scope, condition)
+            }
             Request::Quit => send_one_line!(self, "QUIT"),
         }?;
         Ok(self)
@@ -453,15 +509,10 @@ impl<S: Read + Write + Source> Client<S> {
     }
 
     /// Set the number of (more or less) sentences that should be repeated after a previously paused text is resumed.
-    pub fn set_pause_context(&mut self, scope: ClientScope, value: u8) -> ClientResult<&mut Self> {
+    pub fn set_pause_context(&mut self, scope: ClientScope, value: u32) -> ClientResult<&mut Self> {
         self.send(Request::SetPauseContext(scope, value))
     }
 
-    /// Enable or disable history of received messages.
-    pub fn set_history(&mut self, scope: ClientScope, value: bool) -> ClientResult<&mut Self> {
-        self.send(Request::SetHistory(scope, value))
-    }
-
     /// Enable notification events
     pub fn set_notification(
         &mut self,
@@ -481,6 +532,98 @@ impl<S: Read + Write + Source> Client<S> {
         self.send(Request::End)
     }
 
+    /// Enable or disable history of received messages.
+    pub fn set_history(&mut self, scope: ClientScope, value: bool) -> ClientResult<&mut Self> {
+        self.send(Request::SetHistory(scope, value))
+    }
+
+    /// Get clients in history.
+    pub fn history_get_clients(&mut self) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryGetClients)
+    }
+
+    /// Get client id in the history.
+    pub fn history_get_client_id(&mut self) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryGetClientId)
+    }
+
+    /// Get last message said.
+    pub fn history_get_last(&mut self) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryGetLastMsgId)
+    }
+
+    /// Get a range of client messages.
+    pub fn history_get_client_messages(
+        &mut self,
+        scope: ClientScope,
+        start: u32,
+        number: u32,
+    ) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryGetClientMsgs(scope, start, number))
+    }
+
+    /// Get the id of the last message sent by the client.
+    pub fn history_get_last_message_id(&mut self) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryGetLastMsgId)
+    }
+
+    /// Return the text of an history message.
+    pub fn history_get_message(&mut self, msg_id: MessageId) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryGetMsg(msg_id))
+    }
+
+    /// Get the id of the message the history cursor is pointing to.
+    pub fn history_get_cursor(&mut self) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryCursorGet)
+    }
+
+    /// Set the history cursor position.
+    pub fn history_set_cursor(
+        &mut self,
+        scope: ClientScope,
+        pos: HistoryPosition,
+    ) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryCursorSet(scope, pos))
+    }
+
+    /// Move the cursor position backward or forward.
+    pub fn history_move_cursor(&mut self, direction: CursorDirection) -> ClientResult<&mut Self> {
+        self.send(Request::HistoryCursorMove(direction))
+    }
+
+    /// Speak the message from history.
+    pub fn history_speak(&mut self, msg_id: MessageId) -> ClientResult<&mut Self> {
+        self.send(Request::HistorySpeak(msg_id))
+    }
+
+    /// Sort messages in history.
+    pub fn history_sort(
+        &mut self,
+        direction: SortDirection,
+        key: SortKey,
+    ) -> ClientResult<&mut Self> {
+        self.send(Request::HistorySort(direction, key))
+    }
+
+    /// Set the maximum length of short versions of history messages.
+    pub fn history_set_short_message_length(&mut self, length: u32) -> ClientResult<&mut Self> {
+        self.send(Request::HistorySetShortMsgLength(length))
+    }
+
+    /// Set the ordering of the message types, from the minimum to the maximum.
+    pub fn history_set_ordering(&mut self, ordering: Vec<Ordering>) -> ClientResult<&mut Self> {
+        self.send(Request::HistorySetMsgTypeOrdering(ordering))
+    }
+
+    /// Search in message history.
+    pub fn history_search(
+        &mut self,
+        scope: ClientScope,
+        condition: &str,
+    ) -> ClientResult<&mut Self> {
+        self.send(Request::HistorySearch(scope, condition.to_string()))
+    }
+
     /// Close the connection
     pub fn quit(&mut self) -> ClientResult<&mut Self> {
         self.send(Request::Quit)
@@ -535,15 +678,19 @@ impl<S: Read + Write + Source> Client<S> {
             OK_MSG_CANCELED => Ok(Response::MessageCanceled),
             OK_RECEIVING_DATA => Ok(Response::ReceivingData),
             OK_BYE => Ok(Response::Bye),
-            OK_CLIENT_LIST_SENT => Ok(Response::HistoryClientListSent(lines)),
+            OK_CLIENTS_LIST_SENT => Ok(Response::HistoryClientListSent(parse_typed_lines::<
+                HistoryClientStatus,
+            >(&lines)?)),
             OK_MSGS_LIST_SENT => Ok(Response::HistoryMsgsListSent(lines)),
             OK_LAST_MSG => Ok(Response::HistoryLastMsg(parse_single_value(&lines)?)),
             OK_CUR_POS_RET => Ok(Response::HistoryCurPosRet(parse_single_value(&lines)?)),
             OK_TABLE_LIST_SENT => Ok(Response::TableListSent(lines)),
-            OK_CLIENT_ID_SENT => Ok(Response::HistoryClientIdSent(parse_single_value(&lines)?)),
+            OK_CLIENT_ID_SENT => Ok(Response::HistoryClientIdSent(parse_single_integer(&lines)?)),
             OK_MSG_TEXT_SENT => Ok(Response::MessageTextSent),
             OK_HELP_SENT => Ok(Response::HelpSent(lines)),
-            OK_VOICES_LIST_SENT => Ok(Response::VoicesListSent(parse_synthesis_voices(&lines)?)),
+            OK_VOICES_LIST_SENT => Ok(Response::VoicesListSent(
+                parse_typed_lines::<SynthesisVoice>(&lines)?,
+            )),
             OK_OUTPUT_MODULES_LIST_SENT => Ok(Response::OutputModulesListSent(lines)),
             OK_GET => Ok(Response::Get(parse_single_value(&lines)?)),
             OK_INSIDE_BLOCK => Ok(Response::InsideBlock),
@@ -594,22 +741,51 @@ impl<S: Read + Write + Source> Client<S> {
             .and_then(|lines| parse_single_value(&lines))
     }
 
-    /// Receive integer
+    /// Receive signed 8-bit integer
+    pub fn receive_i8(&mut self) -> ClientResult<u8> {
+        self.receive_string(OK_GET).and_then(|s| {
+            s.parse()
+                .map_err(|_| ClientError::invalid_data("invalid signed integer"))
+        })
+    }
+
+    /// Receive unsigned 8-bit integer
     pub fn receive_u8(&mut self) -> ClientResult<u8> {
-        self.receive_string(OK_GET)
-            .and_then(|s| s.parse().map_err(|_| ClientError::InvalidType))
+        self.receive_string(OK_GET).and_then(|s| {
+            s.parse()
+                .map_err(|_| ClientError::invalid_data("invalid unsigned 8-bit integer"))
+        })
+    }
+
+    /// Receive cursor pos
+    pub fn receive_cursor_pos(&mut self) -> ClientResult<u16> {
+        self.receive_string(OK_CUR_POS_RET).and_then(|s| {
+            s.parse()
+                .map_err(|_| ClientError::invalid_data("invalid unsigned 16-bit integer"))
+        })
     }
 
     /// Receive message id
     pub fn receive_message_id(&mut self) -> ClientResult<MessageId> {
-        self.receive_string(OK_MESSAGE_QUEUED)
-            .and_then(|s| s.parse().map_err(|_| ClientError::InvalidType))
+        let mut lines = Vec::new();
+        match self.receive_answer(&mut lines)?.code {
+            OK_MESSAGE_QUEUED | OK_LAST_MSG => Ok(parse_single_integer(&lines)?),
+            _ => Err(ClientError::invalid_data("not a message id")),
+        }
+    }
+
+    /// Receive client id
+    pub fn receive_client_id(&mut self) -> ClientResult<ClientId> {
+        self.receive_string(OK_CLIENT_ID_SENT).and_then(|s| {
+            s.parse()
+                .map_err(|_| ClientError::invalid_data("invalid client id"))
+        })
     }
 
     /// Receive a list of synthesis voices
     pub fn receive_synthesis_voices(&mut self) -> ClientResult<Vec<SynthesisVoice>> {
         self.receive_lines(OK_VOICES_LIST_SENT)
-            .and_then(|lines| parse_synthesis_voices(&lines))
+            .and_then(|lines| parse_typed_lines::<SynthesisVoice>(&lines))
     }
 
     /// Receive a notification
@@ -617,14 +793,14 @@ impl<S: Read + Write + Source> Client<S> {
         let mut lines = Vec::new();
         crate::protocol::receive_answer(&mut self.input, Some(&mut lines)).and_then(|status| {
             if lines.len() < 2 {
-                Err(ClientError::TruncatedMessage)
+                Err(ClientError::unexpected_eof("event truncated"))
             } else {
                 let message = &lines[0];
                 let client = &lines[1];
                 match status.code {
                     700 => {
                         if lines.len() != 3 {
-                            Err(ClientError::TruncatedMessage)
+                            Err(ClientError::unexpected_eof("index markevent truncated"))
                         } else {
                             let mark = lines[3].to_owned();
                             Ok(Event::index_mark(mark, message, client))
@@ -635,12 +811,18 @@ impl<S: Read + Write + Source> Client<S> {
                     703 => Ok(Event::cancel(message, client)),
                     704 => Ok(Event::pause(message, client)),
                     705 => Ok(Event::resume(message, client)),
-                    _ => Err(ClientError::InvalidType),
+                    _ => Err(ClientError::invalid_data("wrong status code for event")),
                 }
             }
         })
     }
 
+    /// Receive a list of client status from history.
+    pub fn receive_history_clients(&mut self) -> ClientResult<Vec<HistoryClientStatus>> {
+        self.receive_lines(OK_CLIENTS_LIST_SENT)
+            .and_then(|lines| parse_typed_lines::<HistoryClientStatus>(&lines))
+    }
+
     /// Check the result of `set_client_name`.
     pub fn check_client_name_set(&mut self) -> ClientResult<&mut Self> {
         self.check_status(OK_CLIENT_NAME_SET)
