@@ -34,7 +34,10 @@ mod types;
 
 pub mod client;
 pub mod constants;
+#[cfg(unix)]
 pub mod fifo;
+pub mod net;
+pub mod tcp;
 
 #[cfg(any(not(feature = "async-mio"), doc))]
 pub use client::Client;
