@@ -11,7 +11,7 @@ use log::debug;
 use std::io::{self, BufRead, Write};
 use std::str::FromStr;
 
-use crate::types::{ClientError, ClientResult, ClientStatus, EventId, StatusLine, SynthesisVoice};
+use crate::types::{ClientError, ClientResult, ClientStatus, EventId, StatusLine};
 
 macro_rules! invalid_input {
     ($msg:expr) => {
@@ -40,13 +40,27 @@ pub(crate) fn parse_event_id(lines: &[String]) -> ClientResult<EventId> {
     }
 }
 
-pub(crate) fn parse_synthesis_voices(lines: &[String]) -> ClientResult<Vec<SynthesisVoice>> {
-    let mut voices = Vec::new();
-    for name in lines.iter() {
-        let voice = SynthesisVoice::from_str(name.as_str())?;
-        voices.push(voice);
-    }
-    Ok(voices)
+/// Parse single integer value
+pub(crate) fn parse_single_integer<T>(lines: &[String]) -> ClientResult<T>
+where
+    T: FromStr,
+{
+    parse_single_value(lines)?.parse::<T>().map_err(|_| {
+        ClientError::Io(io::Error::new(
+            io::ErrorKind::InvalidData,
+            "invalid integer value",
+        ))
+    })
+}
+
+pub(crate) fn parse_typed_lines<T>(lines: &[String]) -> ClientResult<Vec<T>>
+where
+    T: FromStr<Err = ClientError>,
+{
+    lines
+        .iter()
+        .map(|line| T::from_str(line.as_str()))
+        .collect::<ClientResult<Vec<T>>>()
 }
 
 /// Write lines separated by CRLF.
@@ -120,6 +134,8 @@ mod tests {
 
     use super::{receive_answer, ClientError, ClientResult};
 
+    use crate::types::SynthesisVoice;
+
     #[test]
     fn single_ok_status_line() {
         let mut input = BufReader::new("208 OK CLIENT NAME SET\r\n".as_bytes());
@@ -225,7 +241,7 @@ mod tests {
             .iter()
             .map(|s| s.to_string())
             .collect::<Vec<String>>();
-        let voices = super::parse_synthesis_voices(&lines)?;
+        let voices = super::parse_typed_lines::<SynthesisVoice>(&lines)?;
         assert_eq!(3, voices.len());
         assert_eq!("en", voices[0].name.as_str());
         assert_eq!(Some(String::from("af")), voices[1].language);
