@@ -18,10 +18,10 @@ use strum_macros::Display as StrumDisplay;
 pub type ReturnCode = u16;
 
 /// Message identifier
-pub type MessageId = String;
+pub type MessageId = u32;
 
 /// Client identifier
-pub type ClientId = String;
+pub type ClientId = u32;
 
 /// Message identifiers
 #[derive(Debug, Clone)]
@@ -369,15 +369,18 @@ impl SynthesisVoice {
 }
 
 impl FromStr for SynthesisVoice {
-    type Err = io::Error;
+    type Err = ClientError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let mut iter = s.split('\t');
-        Ok(SynthesisVoice {
-            name: String::from(iter.next().unwrap()),
-            language: SynthesisVoice::parse_none(iter.next()),
-            dialect: SynthesisVoice::parse_none(iter.next()),
-        })
+        match iter.next() {
+            Some(name) => Ok(SynthesisVoice {
+                name: name.to_string(),
+                language: SynthesisVoice::parse_none(iter.next()),
+                dialect: SynthesisVoice::parse_none(iter.next()),
+            }),
+            None => Err(ClientError::unexpected_eof("missing synthesis voice name")),
+        }
     }
 }
 
@@ -402,8 +405,6 @@ impl fmt::Display for StatusLine {
 /// Client error, either I/O error or SSIP error.
 #[derive(ThisError, Debug)]
 pub enum ClientError {
-    #[error("Invalid type")]
-    InvalidType,
     #[error("I/O: {0}")]
     Io(io::Error),
     #[error("Not ready")]
@@ -414,12 +415,27 @@ pub enum ClientError {
     TooFewLines,
     #[error("Too many lines")]
     TooManyLines,
-    #[error("Truncated message")]
-    TruncatedMessage,
     #[error("Unexpected status: {0}")]
     UnexpectedStatus(ReturnCode),
 }
 
+impl ClientError {
+    /// Create I/O error
+    pub(crate) fn io_error(kind: io::ErrorKind, msg: &str) -> Self {
+        Self::Io(io::Error::new(kind, msg))
+    }
+
+    /// Invalid data I/O error
+    pub(crate) fn invalid_data(msg: &str) -> Self {
+        ClientError::io_error(io::ErrorKind::InvalidData, msg)
+    }
+
+    /// Unexpected EOF I/O error
+    pub(crate) fn unexpected_eof(msg: &str) -> Self {
+        ClientError::io_error(io::ErrorKind::UnexpectedEof, msg)
+    }
+}
+
 impl From<io::Error> for ClientError {
     fn from(err: io::Error) -> Self {
         if err.kind() == io::ErrorKind::WouldBlock {
@@ -458,12 +474,123 @@ impl ClientName {
     }
 }
 
+/// Cursor motion in history
+#[derive(StrumDisplay, Debug, Clone)]
+pub enum CursorDirection {
+    #[strum(serialize = "backward")]
+    Backward,
+    #[strum(serialize = "forward")]
+    Forward,
+}
+
+/// Sort direction in history
+#[derive(StrumDisplay, Debug, Clone)]
+pub enum SortDirection {
+    #[strum(serialize = "asc")]
+    Ascending,
+    #[strum(serialize = "desc")]
+    Descending,
+}
+
+/// Property messages are ordered by in history
+#[derive(StrumDisplay, Debug, Clone)]
+pub enum SortKey {
+    #[strum(serialize = "client_name")]
+    ClientName,
+    #[strum(serialize = "priority")]
+    Priority,
+    #[strum(serialize = "message_type")]
+    MessageType,
+    #[strum(serialize = "time")]
+    Time,
+    #[strum(serialize = "user")]
+    User,
+}
+
+/// Sort ordering
+#[derive(StrumDisplay, Debug, Clone)]
+pub enum Ordering {
+    #[strum(serialize = "text")]
+    Text,
+    #[strum(serialize = "sound_icon")]
+    SoundIcon,
+    #[strum(serialize = "char")]
+    Char,
+    #[strum(serialize = "key")]
+    Key,
+}
+
+/// Position in history
+#[derive(Debug, Clone)]
+pub enum HistoryPosition {
+    First,
+    Last,
+    Pos(u16),
+}
+
+impl fmt::Display for HistoryPosition {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            HistoryPosition::First => write!(f, "first"),
+            HistoryPosition::Last => write!(f, "last"),
+            HistoryPosition::Pos(n) => write!(f, "pos {}", n),
+        }
+    }
+}
+
+/// History client status
+#[derive(Debug, PartialEq)]
+pub struct HistoryClientStatus {
+    pub id: ClientId,
+    pub name: String,
+    pub connected: bool,
+}
+
+impl HistoryClientStatus {
+    pub fn new(id: ClientId, name: &str, connected: bool) -> Self {
+        Self {
+            id,
+            name: name.to_string(),
+            connected,
+        }
+    }
+}
+
+impl FromStr for HistoryClientStatus {
+    type Err = ClientError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let mut iter = s.splitn(3, ' ');
+        match iter.next() {
+            Some("") => Err(ClientError::unexpected_eof("expecting client id")),
+            Some(client_id) => match client_id.parse::<u32>() {
+                Ok(id) => match iter.next() {
+                    Some(name) => match iter.next() {
+                        Some(status) if status == "0" => {
+                            Ok(HistoryClientStatus::new(id, name, false))
+                        }
+                        Some(status) if status == "1" => {
+                            Ok(HistoryClientStatus::new(id, name, true))
+                        }
+                        Some(_) => Err(ClientError::invalid_data("invalid client status")),
+                        None => Err(ClientError::unexpected_eof("expecting client status")),
+                    },
+                    None => Err(ClientError::unexpected_eof("expecting client name")),
+                },
+                Err(_) => Err(ClientError::invalid_data("invalid client id")),
+            },
+            None => Err(ClientError::unexpected_eof("expecting client id")),
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
 
+    use std::io;
     use std::str::FromStr;
 
-    use super::{MessageScope, SynthesisVoice};
+    use super::{ClientError, HistoryClientStatus, HistoryPosition, MessageScope, SynthesisVoice};
 
     #[test]
     fn parse_synthesis_voice() {
@@ -485,9 +612,42 @@ mod tests {
     fn format_message_scope() {
         assert_eq!("self", format!("{}", MessageScope::Last).as_str());
         assert_eq!("all", format!("{}", MessageScope::All).as_str());
+        assert_eq!("123", format!("{}", MessageScope::Message(123)).as_str());
+    }
+
+    #[test]
+    fn format_history_position() {
+        assert_eq!("first", format!("{}", HistoryPosition::First).as_str());
+        assert_eq!("last", format!("{}", HistoryPosition::Last).as_str());
+        assert_eq!("pos 15", format!("{}", HistoryPosition::Pos(15)).as_str());
+    }
+
+    #[test]
+    fn parse_history_client_status() {
         assert_eq!(
-            "123",
-            format!("{}", MessageScope::Message("123".to_string())).as_str()
+            HistoryClientStatus::new(10, "joe:speechd_client:main", false),
+            HistoryClientStatus::from_str("10 joe:speechd_client:main 0").unwrap()
         );
+        assert_eq!(
+            HistoryClientStatus::new(11, "joe:speechd_client:main", true),
+            HistoryClientStatus::from_str("11 joe:speechd_client:main 1").unwrap()
+        );
+        for line in &[
+            "9 joe:speechd_client:main xxx",
+            "xxx joe:speechd_client:main 1",
+        ] {
+            match HistoryClientStatus::from_str(line) {
+                Ok(_) => panic!("parsing should have failed"),
+                Err(ClientError::Io(err)) if err.kind() == io::ErrorKind::InvalidData => (),
+                Err(_) => panic!("expecting error 'invalid data' parsing \"{}\"", line),
+            }
+        }
+        for line in &["8 joe:speechd_client:main", "8", ""] {
+            match HistoryClientStatus::from_str(line) {
+                Ok(_) => panic!("parsing should have failed"),
+                Err(ClientError::Io(err)) if err.kind() == io::ErrorKind::UnexpectedEof => (),
+                Err(_) => panic!("expecting error 'unexpected EOF' parsing \"{}\"", line),
+            }
+        }
     }
 }
